var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __reExport = (target, mod, secondTarget) => (__copyProps(target, mod, "default"), secondTarget && __copyProps(secondTarget, mod, "default"));

// node_modules/@girs/gnome-shell/dist/extensions/extension.js
var extension_exports = {};
__reExport(extension_exports, extension_star);
import * as extension_star from "resource:///org/gnome/shell/extensions/extension.js";

// node_modules/@girs/gnome-shell/dist/ui/main.js
var main_exports = {};
__reExport(main_exports, main_star);
import * as main_star from "resource:///org/gnome/shell/ui/main.js";

// src/Utils.ts
var Utils = class {
  static ICONS = {
    INDICATOR_DEFAULT: "/icons/icon-symbolic.svg",
    INDICATOR_PAUSED: "/icons/icon-paused-symbolic.svg",
    INDICATOR_PLAYING: "/icons/icon-playing-symbolic.svg",
    POPUP_PLAY: "media-playback-start-symbolic",
    POPUP_STOP: "media-playback-stop-symbolic",
    POPUP_PAUSE: "media-playback-pause-symbolic"
  };
  static debug(...message) {
    console.log("[ QUICK LOFI DEBUG ] >>> ", ...message);
  }
  static isCurrentRadioPlaying(settings, radioID) {
    const currentRadioPlaying = settings.get_string("current-radio-playing");
    return currentRadioPlaying.length > 0 && radioID === currentRadioPlaying;
  }
  static generateNanoIdWithSymbols(size) {
    const characters = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789!@#$%^&*()_+-=[]{}|;:,.<>?";
    const charactersLength = characters.length;
    let result = "";
    for (let i = 0; i < size; i++) {
      result += characters.charAt(Math.floor(Math.random() * charactersLength));
    }
    return result;
  }
};

// node_modules/@girs/gnome-shell/dist/ui/slider.js
var slider_exports = {};
__reExport(slider_exports, slider_star);
import * as slider_star from "resource:///org/gnome/shell/ui/slider.js";

// node_modules/@girs/gnome-shell/dist/ui/panelMenu.js
var panelMenu_exports = {};
__reExport(panelMenu_exports, panelMenu_star);
import * as panelMenu_star from "resource:///org/gnome/shell/ui/panelMenu.js";

// node_modules/@girs/gnome-shell/dist/ui/popupMenu.js
var popupMenu_exports = {};
__reExport(popupMenu_exports, popupMenu_star);
import * as popupMenu_star from "resource:///org/gnome/shell/ui/popupMenu.js";

// src/Player.ts
import GLib from "gi://GLib";
import Gio2 from "gi://Gio";
var Player = class {
  constructor(_settings) {
    this._settings = _settings;
  }
  _mpvSocket = "/tmp/quicklofi-socket";
  _isCommandRunning = false;
  _process = null;
  debounceTimeout = null;
  initVolumeControl() {
    this._settings.connect("changed::volume", (settings, key) => {
      if (this._process !== null && !this._isCommandRunning) {
        const volume = settings.get_int(key);
        const command = this.createCommand({
          command: ["set_property", "volume", volume]
        });
        this.sendCommandToMpvSocket(command);
      }
    });
  }
  stopPlayer() {
    if (this._process !== null) {
      this._process.force_exit();
      this._process = null;
      return;
    }
  }
  playPause() {
    const playPauseCommand = this.createCommand({ command: ["cycle", "pause"] });
    this.sendCommandToMpvSocket(playPauseCommand, 0);
  }
  startPlayer(radio) {
    this.stopPlayer();
    try {
      const [, argv] = GLib.shell_parse_argv(
        `mpv --volume=${this._settings.get_int("volume")} --input-ipc-server=/tmp/quicklofi-socket --loop-playlist=force --no-video --ytdl-format='best*[vcodec=none]' --ytdl-raw-options-add='force-ipv4=' ${radio.radioUrl}`
      );
      this._process = Gio2.Subprocess.new(argv, Gio2.SubprocessFlags.NONE);
    } catch (e) {
      this._process = null;
      main_exports.notifyError(
        "MPV not found",
        "Did you have mpv installed?\nhttps://github.com/EuCaue/gnome-shell-extension-quick-lofi?tab=readme-ov-file#dependencies"
      );
    }
  }
  createCommand(command) {
    const cmd = `echo '${JSON.stringify(command)}'`;
    return cmd;
  }
  sendCommandToMpvSocket(mpvCommand, debounceInterval = 550) {
    if (this.debounceTimeout !== null) {
      GLib.Source.remove(this.debounceTimeout);
    }
    this.debounceTimeout = GLib.timeout_add(GLib.PRIORITY_DEFAULT, debounceInterval, () => {
      this._isCommandRunning = true;
      const socatCommand = ["|", "socat", "-", this._mpvSocket];
      const [success, _] = GLib.spawn_async(
        null,
        ["/bin/sh", "-c", mpvCommand + " " + socatCommand.join(" ")],
        null,
        GLib.SpawnFlags.SEARCH_PATH,
        null
      );
      this._isCommandRunning = false;
      if (!success) {
        main_exports.notifyError(
          "Socat not found",
          "Did you have socat installed?\nhttps://github.com/EuCaue/gnome-shell-extension-quick-lofi?tab=readme-ov-file#dependencies"
        );
      }
      this.debounceTimeout = null;
      return GLib.SOURCE_REMOVE;
    });
  }
};

// src/Indicator.ts
import Gio3 from "gi://Gio";
import St from "gi://St";
import GObject from "gi://GObject";
import GLib2 from "gi://GLib";
var _Indicator = class _Indicator extends panelMenu_exports.Button {
  mpvPlayer;
  _activeRadioPopupItem = null;
  _radios;
  _icon;
  _extension;
  constructor(ext) {
    super(0, "Quick Lofi");
    this._extension = ext;
    this.mpvPlayer = new Player(this._extension._settings);
    this.mpvPlayer.initVolumeControl();
    this._icon = new St.Icon({
      gicon: Gio3.icon_new_for_string(this._extension.path + Utils.ICONS.INDICATOR_DEFAULT),
      iconSize: 20,
      styleClass: "system-status-icon indicator-icon"
    });
    this.add_child(this._icon);
    this._createMenu();
    this._bindSettingsChangeEvents();
    this._handleButtonClick();
  }
  _createRadios() {
    const radios = this._extension._settings.get_strv("radios");
    radios.forEach((entry) => {
      const [radioName, radioUrl, id] = entry.split(" - ");
      this._radios.push({ radioName, radioUrl, id });
    });
  }
  _handlePopupMaxHeight() {
    const isPopupMaxHeightSet = this._extension._settings.get_boolean("set-popup-max-height");
    const popupMaxHeight = this._extension._settings.get_string("popup-max-height");
    const styleString = isPopupMaxHeightSet ? popupMaxHeight : "auto";
    this.menu.box.style = `
        max-height: ${styleString};
      `;
  }
  _bindSettingsChangeEvents() {
    this._extension._settings.connect("changed", (_, key) => {
      if (key === "radios") {
        this._createMenu();
      }
    });
    this._extension._settings.connect("changed::set-popup-max-height", () => {
      this._handlePopupMaxHeight();
    });
    this._extension._settings.connect("changed::popup-max-height", () => {
      this._handlePopupMaxHeight();
    });
  }
  _updateIndicatorIcon({ playing }) {
    const extPath = this._extension.path;
    const icon = `INDICATOR_${playing.toUpperCase()}`;
    const iconPath = `${extPath}/${Utils.ICONS[icon]}`;
    const gicon = Gio3.icon_new_for_string(iconPath);
    this._icon.set_gicon(gicon);
  }
  _togglePlayingStatus(child, radioID, mouseButton) {
    const isRightClickOnActiveRadio = child === this._activeRadioPopupItem && mouseButton === 3;
    const isLeftClickOnActiveRadio = child === this._activeRadioPopupItem && mouseButton === 1;
    if (isRightClickOnActiveRadio) {
      this.mpvPlayer.stopPlayer();
      this._activeRadioPopupItem.setIcon(Gio3.icon_new_for_string(Utils.ICONS.POPUP_PLAY));
      this._activeRadioPopupItem = null;
      this._updateIndicatorIcon({ playing: "default" });
      this._extension._settings.set_string("current-radio-playing", "");
      child.set_style("font-weight: normal");
      return;
    }
    if (isLeftClickOnActiveRadio) {
      const currentState = this._activeRadioPopupItem.get_child_at_index(0).icon_name;
      const isPlaying = currentState === Utils.ICONS.POPUP_STOP;
      this._activeRadioPopupItem.setIcon(
        Gio3.icon_new_for_string(isPlaying ? Utils.ICONS.POPUP_PAUSE : Utils.ICONS.POPUP_STOP)
      );
      this._activeRadioPopupItem.set_style(`font-weight: ${isPlaying ? "bold" : "normal"}`);
      this._updateIndicatorIcon({ playing: isPlaying ? "paused" : "playing" });
      this.mpvPlayer.playPause();
      return;
    }
    const currentRadio = this._radios.find((radio) => radio.id === radioID);
    if (this._activeRadioPopupItem) {
      this._activeRadioPopupItem.setIcon(Gio3.icon_new_for_string(Utils.ICONS.POPUP_PLAY));
      this._activeRadioPopupItem.set_style("font-weight: normal");
      this._updateIndicatorIcon({ playing: "default" });
    }
    this.mpvPlayer.startPlayer(currentRadio);
    this._updateIndicatorIcon({ playing: "playing" });
    child.setIcon(Gio3.icon_new_for_string(Utils.ICONS.POPUP_STOP));
    child.set_style("font-weight: bold");
    this._extension._settings.set_string("current-radio-playing", radioID);
    this._activeRadioPopupItem = child;
  }
  _handleButtonClick() {
    this.connect("button-press-event", (_, event) => {
      const RIGHT_CLICK = 3;
      if (event.get_button() === RIGHT_CLICK) {
        this.menu.close(false);
        this._extension.openPreferences();
        return;
      }
    });
  }
  _createMenu() {
    this._activeRadioPopupItem = null;
    this.menu.box.destroy_all_children();
    this._radios = [];
    this._createRadios();
    this._createMenuItems();
  }
  _createVolumeSlider(popup) {
    const separator = new popupMenu_exports.PopupSeparatorMenuItem();
    const volumeLevel = this._extension._settings.get_int("volume");
    const volumePopupItem = new popupMenu_exports.PopupBaseMenuItem({ reactive: false });
    const volumeBoxLayout = new St.BoxLayout({ vertical: true, x_expand: true });
    const volumeSlider = new slider_exports.Slider(volumeLevel / 100);
    volumeSlider.connect("notify::value", (slider) => {
      const currentVolume = (slider.value * 100).toFixed(0);
      volumeLabel.text = `Volume: ${currentVolume}`;
      this._extension._settings.set_int("volume", Number(currentVolume));
    });
    const volumeLabel = new St.Label({ text: `Volume: ${volumeSlider.value * 100}` });
    volumeBoxLayout.add_child(volumeLabel);
    volumeBoxLayout.add_child(volumeSlider);
    volumePopupItem.add_child(volumeBoxLayout);
    popup.addMenuItem(separator);
    popup.addMenuItem(volumePopupItem);
  }
  _createMenuItems() {
    const scrollView = new St.ScrollView();
    const popupSection = new popupMenu_exports.PopupMenuSection();
    scrollView.add_child(popupSection.actor);
    this._radios.forEach((radio) => {
      const isRadioPlaying = Utils.isCurrentRadioPlaying(this._extension._settings, radio.id);
      const menuItem = new popupMenu_exports.PopupImageMenuItem(
        radio.radioName,
        Gio3.icon_new_for_string(isRadioPlaying ? Utils.ICONS.POPUP_STOP : Utils.ICONS.POPUP_PLAY)
      );
      if (isRadioPlaying)
        menuItem.set_style("font-weight: bold");
      menuItem.connect("activate", (item, event) => {
        const mouseButton = event.get_button();
        this._togglePlayingStatus(item, radio.id, mouseButton);
      });
      popupSection.addMenuItem(menuItem);
    });
    this._createVolumeSlider(popupSection);
    this.menu.box.add_child(scrollView);
    this._handlePopupMaxHeight();
  }
  dispose() {
    Utils.debug("extension disabled");
    this._extension._settings.set_string("current-radio-playing", "");
    this.mpvPlayer.stopPlayer();
    if (this.mpvPlayer.debounceTimeout) {
      GLib2.Source.remove(this.mpvPlayer.debounceTimeout);
      this.mpvPlayer.debounceTimeout = null;
    }
    this.destroy();
  }
};
GObject.registerClass(_Indicator);
var Indicator = _Indicator;

// src/extension.ts
var QuickLofi = class extends extension_exports.Extension {
  _indicator = null;
  _settings = null;
  constructor(props) {
    super(props);
  }
  _migrateRadios() {
    const radios = this._settings.get_strv("radios");
    const updatedRadios = radios.map((radio) => {
      if (radio.split(" - ").length === 3) {
        return radio;
      }
      if (radio.includes(" - ")) {
        const [name, url] = radio.split(" - ");
        const id = Utils.generateNanoIdWithSymbols(10);
        return `${name} - ${url} - ${id}`;
      }
    });
    if (JSON.stringify(radios) === JSON.stringify(updatedRadios))
      return;
    this._settings.set_strv("radios", updatedRadios);
  }
  enable() {
    Utils.debug("extension enabled");
    this._settings = this.getSettings();
    this._settings.set_string("current-radio-playing", "");
    this._migrateRadios();
    this._indicator = new Indicator(this);
    main_exports.panel.addToStatusArea(this.uuid, this._indicator);
  }
  disable() {
    this._indicator.dispose();
    this._indicator = null;
    this._settings = null;
  }
};
export {
  QuickLofi as default
};
